#!/usr/bin/python

# Anne S. Warlaumont

# A simple computational model of the social feedback loop facilitating child speech development
# My adaptation to do something in between Rescorla-Wagner matching and the too-fast-to-converge traditional Q-learning

import random

def gradualRL(ASDadult,ASDvol): # ASD can either be 0 (which models typical development) or 1 (which models autism)
    
        alpha=.000003 # fixed parameter that determines learning rate
        numdays=365*3
        #numdays=1
    
        # Set the initial Qs:
        Psp=.5 # speech-related
        Pnsp=.5 # non-speech-related
        
        # Set the probability of adult response given that the child voc is speech-related or not speech-related
        # The values below are drawn from LENA data
        if ASDadult==0:
            Prsp= .209 #.225
            Prnsp= .144 #.171
        else:
            Prsp= .197 #.193
            Prnsp= .148 #.153
        sumPrspPrnsp=Prsp+Prnsp
        Prsp=Prsp/sumPrspPrnsp # Probability of adult response given the child vocalization is speech-related
        Prnsp=Prnsp/sumPrspPrnsp # Probability of adult response given the child vocalization is not speech-related
        
        # Set the number of vocalizations per day and total number of iterations
        if ASDvol==0:
            vocsperday=2592 #2499
        else:
            vocsperday=2123 #1979
        numvocs=vocsperday*numdays # total number of iterations
        
        # Initialize the records of the child's and adult's behaviors:
        sprec=[0]*numvocs
        resprec=[0]*numvocs
        nsp=0
        nnsp=0
        lastresptosp = False; # Whether or not the last child speech-related vocalization was responded to
        n_c_sp_r_sp = 0
        n_c_r_sp = 0
        n_c_sp_nor_sp = 0
        n_c_nor_sp = 0 
        
        for voc in range(numvocs):
        
            # Get the child behavior:
            rc=random.random()
            
            if rc <= Psp/(Psp+Pnsp):
                sp=1
            else:
                sp=0
        
            if lastresptosp:
                n_c_r_sp += 1 # number of child vocalizations when the prevous child speech-related vocalization did receive a response
                if sp==1:
                    nsp+=1 # number of child vocalizations that were speech-related
                    n_c_sp_r_sp += 1 # number of child speech-related vocalizations when the previous child speech-related vocalization did receive a response
                else: 
                    nnsp+=1 # number of child vocalizations that were not speech-related
            else:
                n_c_nor_sp += 1 # number of child vocalizations when the previous child speech-related vocalization did NOT receive a response
                if sp==1:
                    nsp+=1 # number of child vocalizations that were speech-related
                    n_c_sp_nor_sp += 1 # number of child speech-related vocalizations when the previous child speech-related vocalization did NOT receive a response
                else:
                    nnsp+=1 # number of child vocalizations that were not speech-related
        
            # Get the adult behavior:
            ra=random.random()
            if (sp==1 and ra<=Prsp) or (sp==0 and ra<=Prnsp):
                resp=1
            else:
                resp=0
            sprec[voc]=sp
            resprec[voc]=resp
            
            if (sp==1):
                # Reset the record of whether the last speech-related child voc received a response
                if (resp==1):
                    lastresptosp = True
                else:
                    lastresptosp = False
           
            if sp==1 and resp==1:
                Psp=Psp+alpha
            elif sp==0 and resp==1:
                Pnsp=Pnsp+alpha

            sumPspPnsp = Psp + Pnsp
            Psp=Psp/sumPspPnsp
            Pnsp=Pnsp/sumPspPnsp
            childcontonadult = n_c_sp_r_sp/(n_c_r_sp+.01) - (n_c_sp_nor_sp)/(n_c_nor_sp+.01)
    
        lastdaysprate=float(sum(sprec[(numvocs-vocsperday+1):numvocs]))/vocsperday
        
        childcontonadult = n_c_sp_r_sp/(n_c_r_sp+.01) - (n_c_sp_nor_sp)/(n_c_nor_sp+.01)
    
        print
        print "Psp: ", Psp
        print "Pnsp: ", Pnsp
        #print "nsp: ", nsp
        #print "nnsp: ", nnsp
        print "proportion speech-related during the last day: ", lastdaysprate
        print "child contingency on adult response: ", childcontonadult
    
        return Psp, Pnsp, nsp, nnsp, lastdaysprate, childcontonadult
    	
				